// ---------------------------------------------------------------------------
// Created by Francisco Malpartida on 16/04/11.
// Copyright 2011 - Under creative commons license 3.0:
//        Attribution-NonCommercial-ShareAlike CC BY-NC-SA
//
// This software is furnished "as is", without technical support, and with no 
// warranty, express or implied, as to its usefulness for any purpose.
//
// Thread Safe: No
// Extendable: No
//
// @file stepper.cpp
// File description here
// 
// @brief Stepper driver
//
// @author F. Malpartida
// ---------------------------------------------------------------------------
#include <Arduino.h>
#include "stepper.h"


static long toSteps(float steps_per_unit, float targetUnits)
{
   return steps_per_unit * targetUnits;
}

stepper::stepper ( uint8_t stepPin, uint8_t dirPin, uint8_t enPin, uint8_t pulseW )
{
   _stepPin  = stepPin;
   _dirPin   = dirPin;
   _enPin    = enPin;
   _pulseW   = pulseW;
   _numSteps = 0;
   
   current_units = 0.0;
   target_units  = 0.0;
   
   pinMode ( _stepPin, OUTPUT );
   pinMode ( _dirPin, OUTPUT );
   pinMode ( _enPin, OUTPUT );
   
   digitalWrite ( _stepPin, LOW );
   digitalWrite ( _enPin, DISABLE );
   digitalWrite ( _dirPin, stepper::LEFT );
}

stepper::stepper ( uint8_t stepPin, uint8_t dirPin, uint8_t enPin, uint8_t pulseW,
                  float units )
{
   _stepPin  = stepPin;
   _dirPin   = dirPin;
   _enPin    = enPin;
   _pulseW   = pulseW;
   _numSteps = 0;
   
   current_units = 0.0;
   target_units  = 0.0;
   
   pinMode ( _stepPin, OUTPUT );
   pinMode ( _dirPin, OUTPUT );
   pinMode ( _enPin, OUTPUT );
   
   digitalWrite ( _stepPin, LOW );
   digitalWrite ( _enPin, DISABLE );
   digitalWrite ( _dirPin, stepper::LEFT );
   
   _units = units;
}

void stepper::runStep ( )
{
   if ( _numSteps > 0 )
   {
      digitalWrite ( _stepPin, HIGH );
      delayMicroseconds ( _pulseW );
      digitalWrite ( _stepPin, LOW );
      _numSteps--;
   }
}

void stepper::step ( )
{
   digitalWrite ( _stepPin, HIGH );
   delayMicroseconds ( _pulseW );
   digitalWrite ( _stepPin, LOW );
}


void stepper::setSteps ( uint16_t numSteps )
{
   _numSteps = numSteps;
}

uint16_t stepper::getSteps ( void )
{
   return ( _numSteps );
}

void stepper::direction ( t_dir direction )
{
   digitalWrite (_dirPin, direction );
}

void stepper::enable ( void )
{
   digitalWrite ( _enPin, ENABLE );
}

void stepper::disable ( void )
{
   digitalWrite ( _enPin, DISABLE );
}

void stepper::calculateMove ( void )
{
   delta_units = abs(target_units - current_units);
   
   //set our steps current, target, and delta
   current_steps = toSteps(_units, current_units);
   
   target_steps = toSteps(_units, target_units);
   
   delta_steps = abs(target_steps - current_steps);
   
   //what is our direction
   _direction = (target_units >= current_units);
}


void stepper::setUnits ( float units )
{
   _units = units;
}

void stepper::setPosition ( float position )
{
   current_units = position;
   calculateMove();
}

float stepper::getStepPosition ( void )
{
   return ( current_steps );
}

void stepper::setTarget ( float target )
{
   target_units = target;
}

float stepper::getStepTarget ( void )
{
   return ( target_steps );
}