// ---------------------------------------------------------------------------
// Created by Francisco Malpartida on 16/04/11.
// Copyright 2011 - Under creative commons license 3.0:
//        Attribution-NonCommercial-ShareAlike CC BY-NC-SA
//
// This software is furnished "as is", without technical support, and with no 
// warranty, express or implied, as to its usefulness for any purpose.
//
// Thread Safe: No
// Extendable: No
//
// @file stepper.h
// File description here
// 
// @brief Stepper driver
//
// @author F. Malpartida
// ---------------------------------------------------------------------------
#ifndef _STEPPER_H_
#define _STEPPER_H_
#include <stdint.h>
#include "configMe.h"

// Stepper class definition
// ------------------------
class stepper
{
public:
   typedef enum { LEFT = 0, RIGHT = 1 } t_dir;
   
   stepper ( uint8_t stepPin, uint8_t dirPin, uint8_t enPin, uint8_t pulseW );
   stepper ( uint8_t stepPin, uint8_t dirPin, uint8_t enPin, uint8_t pulseW, float units );
   void     step ( void );
   void     runStep ( void );
   void     direction ( t_dir direction );
   void     setSteps ( uint16_t numSteps );
   void     enable ( void );
   void     disable ( void );
   uint16_t getSteps ( void );
   
   void setUnits ( float units );   // Sets steps per mm
   void setPosition ( float position );
   float getStepPosition ( void );
   void setTarget ( float target );
   float getStepTarget ( void );
   
private:
   void calculateMove ( void );
   
   uint8_t  _stepPin;
   uint8_t  _dirPin;
   uint8_t  _enPin;
   uint8_t  _pulseW;    // pulse width in useconds
   uint16_t _numSteps;
   uint8_t  _direction;
   long     _stepCount;
   
   float current_units;
   float target_units;
   float delta_units;

   float current_steps;
   float target_steps;
   float delta_steps;
   
   float _units;       // Convertion units: steps per mm
};

#endif
